@@ -10,7 +10,7 @@ package Sidef::Types::Array::Array {
       );
 
     use overload
-      q{""}   => \&dump,
+      q{""}   => \&_dump,
       q{0+}   => sub { scalar(@{$_[0]}) },
       q{bool} => sub { scalar(@{$_[0]}) };
 
@@ -129,7 +129,7 @@ package Sidef::Types::Array::Array {
         if ($operator eq '') {
             foreach my $i (@{$self}) {
                 foreach my $j (@arg) {
-                    CORE::push(@array, $self->new($i, $j));
+                    CORE::push(@array, $self->new([$i, $j]));
                 }
             }
         }
@@ -197,6 +197,17 @@ package Sidef::Types::Array::Array {
         $self->new(\@array);
     }
 
+    sub part {
+        my ($self, $num) = @_;
+
+        my @first = @$self;
+        my @second = splice(@first, CORE::int($num));
+
+        ($self->new(\@first), $self->new(\@second));
+    }
+
+    *partition = \&part;
+
     sub or {
         my ($self, $array) = @_;
 
@@ -327,8 +338,8 @@ package Sidef::Types::Array::Array {
         my ($self, $arg) = @_;
 
         UNIVERSAL::isa($arg, 'ARRAY')
-          ? $self->new(@{$self}, @{$arg})
-          : $self->new(@{$self}, $arg);
+          ? $self->new([@{$self}, @{$arg}])
+          : $self->new([@{$self}, $arg]);
     }
 
     sub levenshtein {
@@ -434,7 +445,7 @@ package Sidef::Types::Array::Array {
             my @c = (0 .. $k - 1);
 
             while (1) {
-                $block->run($self->new(@{$self}[@c]));
+                $block->run($self->new([@{$self}[@c]]));
                 next if ($c[$k - 1]++ < $n - 1);
                 my $i = $k - 2;
                 $i-- while ($i >= 0 && $c[$i] >= $n - ($k - $i));
@@ -667,7 +678,7 @@ package Sidef::Types::Array::Array {
             }
 
             my $from = @{$self} - CORE::int($arg);
-            return $self->new(@{$self}[($from < 0 ? 0 : $from) .. $#{$self}]);
+            return $self->new([@{$self}[($from < 0 ? 0 : $from) .. $#{$self}]]);
         }
 
         @{$self} ? $self->[-1] : ();
@@ -976,9 +987,7 @@ package Sidef::Types::Array::Array {
         my $i = 0;
         Sidef::Types::Block::Block->new(
             code => sub {
-                my $result = exists($self->[$i]) ? $self->[$i] : return;
-                ++$i;
-                $result;
+                $self->[$i++];
             }
         );
     }
@@ -1144,7 +1153,7 @@ package Sidef::Types::Array::Array {
 
     sub shuffle {
         my ($self) = @_;
-        $self->new(List::Util::shuffle(@{$self}));
+        $self->new([List::Util::shuffle(@{$self})]);
     }
 
     sub best_shuffle {
@@ -1174,13 +1183,12 @@ package Sidef::Types::Array::Array {
     }
 
     sub reduce {
-        my ($self, $obj) = @_;
+        my ($self, $obj, $initial) = @_;
 
         if (ref($obj) eq 'Sidef::Types::Block::Block') {
             (my $end = $#{$self}) >= 0 || return;
-
-            my $x = $self->[0];
-            foreach my $i (1 .. $end) {
+            my ($beg, $x) = defined($initial) ? (0, $initial) : (1, $self->[0]);
+            foreach my $i ($beg .. $end) {
                 $x = $obj->run($x, $self->[$i]);
             }
 
@@ -1255,6 +1263,8 @@ package Sidef::Types::Array::Array {
             [map { Sidef::Types::Array::Pair->new(Sidef::Types::Number::Number->_new_uint($_), $self->[$_]) } 0 .. $#{$self}]);
     }
 
+    *kv = \&pairs;
+
     sub insert {
         my ($self, $i, @objects) = @_;
 
@@ -1842,7 +1852,7 @@ package Sidef::Types::Array::Array {
 
     sub to_list { @{$_[0]} }
 
-    sub dump {
+    sub _dump {
 
         my %addr;    # keeps track of dumped objects
 
@@ -1854,27 +1864,24 @@ package Sidef::Types::Array::Array {
             exists($addr{$refaddr})
               and return $addr{$refaddr};
 
-            my $str = Sidef::Types::String::String->new("Array(#`($refaddr)...)");
-            $addr{$refaddr} = $str;
+            $addr{$refaddr} = "Array(#`($refaddr)...)";
 
-            $$str = (
-                '[' . CORE::join(
-                    ', ',
-                    map {
-                        my $item = defined($obj->[$_]) ? $obj->[$_] : 'nil';
-                        ref($item) && defined(UNIVERSAL::can($item, 'dump')) ? $item->dump() : $item;
-                      } 0 .. $#{$obj}
-                  )
-                  . ']'
-            );
+            my $s;
 
-            $str;
+            '['
+              . CORE::join(', ',
+                           map { ref($_) && ($s = UNIVERSAL::can($_, 'dump')) ? $s->($_) : defined($_) ? $_ : 'nil' } @$obj)
+              . ']';
         };
 
         local *Sidef::Types::Array::Array::dump = $sub;
         $sub->($_[0]);
     }
 
+    sub dump {
+        Sidef::Types::String::String->new($_[0]->_dump);
+    }
+
     sub to_s {
         my ($self) = @_;
         Sidef::Types::String::String->new(CORE::join(' ', @{$self}));
