@@ -12,8 +12,9 @@ package Sidef::Types::Glob::File {
     sub new {
         my (undef, $file) = @_;
         if (@_ > 2) {
+            shift(@_);
             state $x = require File::Spec;
-            $file = File::Spec->catfile(map { "$_" } @_[1 .. $#_]);
+            $file = File::Spec->catfile(map { "$_" } @_);
         }
         elsif (ref($file) && ref($file) ne 'SCALAR') {
             $file = "$file";
@@ -34,25 +35,11 @@ package Sidef::Types::Glob::File {
         foreach my $name (@Fcntl::EXPORT, @Fcntl::EXPORT_OK) {
             $name =~ /^[a-z]/i or next;
             *{__PACKAGE__ . '::' . $name} = sub {
-                $cache{$name} //= Sidef::Types::Number::Number::_new_uint(&{'Fcntl::' . $name});
+                $cache{$name} //= Sidef::Types::Number::Number->_new_uint(&{'Fcntl::' . $name});
             };
         }
     }
 
-    sub touch {
-        my ($self, @args) = @_;
-
-        if (not ref($self)) {
-            $self = $self->new(shift @args);
-        }
-
-        $self->open('>>', @args);
-    }
-
-    *make   = \&touch;
-    *mkfile = \&touch;
-    *create = \&touch;
-
     sub size {
         ref($_[0]) || shift(@_);
         my ($self) = @_;
@@ -99,7 +86,9 @@ package Sidef::Types::Glob::File {
     sub readlink {
         ref($_[0]) || shift(@_);
         my ($self) = @_;
-        Sidef::Types::String::String->new(CORE::readlink("$self"));
+        my $link = "$self";
+        my $class = (-d $link) ? 'Sidef::Types::Glob::Dir' : __PACKAGE__;
+        $class->new(CORE::readlink($link));
     }
 
     *read_link = \&readlink;
@@ -265,14 +254,11 @@ package Sidef::Types::Glob::File {
     *is_abs = \&is_absolute;
 
     sub abs_name {
-        my ($self) = @_;
-
-        unless (ref($self)) {
-            $self = $self->new($_[1]);
-        }
+        my $class = ref($_[0]) || shift(@_);
+        my ($self, $base) = @_;
 
         state $x = require File::Spec;
-        $self->new(File::Spec->rel2abs("$self"));
+        $class->new(File::Spec->rel2abs("$self", defined($base) ? "$base" : ()));
     }
 
     *abs     = \&abs_name;
@@ -280,20 +266,27 @@ package Sidef::Types::Glob::File {
     *rel2abs = \&abs_name;
 
     sub rel_name {
+        my $class = ref($_[0]) || shift(@_);
         my ($self, $base) = @_;
 
-        unless (ref($self)) {
-            ($self, $base) = ($self->new($base), $_[2]);
-        }
-
         state $x = require File::Spec;
-        $self->new(File::Spec->rel2abs("$self", defined($base) ? "$base" : ()));
+        $class->new(File::Spec->rel2abs("$self", defined($base) ? "$base" : ()));
     }
 
     *rel     = \&rel_name;
     *relname = \&rel_name;
     *abs2rel = \&rel_name;
 
+    sub abs_path {
+        my $class = ref($_[0]) || shift(@_);
+        my ($self) = @_;
+
+        state $x = require Cwd;
+        $class->new(Cwd::abs_path("$self"));
+    }
+
+    *realpath = \&abs_path;
+
     sub rename {
         ref($_[0]) || shift(@_);
         my ($self, $file) = @_;
@@ -425,7 +418,7 @@ package Sidef::Types::Glob::File {
 
         my $success = CORE::open(my $fh, $mode, ref($self) eq __PACKAGE__ ? $$self : "$self");
         my $error   = $!;
-        my $fh_obj  = Sidef::Types::Glob::FileHandle->new(fh => $fh, self => $self);
+        my $fh_obj  = Sidef::Types::Glob::FileHandle->new($fh, $self);
 
         if (defined $fh_ref) {
             ${$fh_ref} = $fh_obj;
@@ -441,50 +434,52 @@ package Sidef::Types::Glob::File {
         $success ? $fh_obj : ();
     }
 
+    sub touch {
+        ref($_[0]) || shift(@_);
+        my ($self, @args) = @_;
+        Sidef::Types::Glob::File::open($self, '>>', @args);
+    }
+
+    *make   = \&touch;
+    *mkfile = \&touch;
+    *create = \&touch;
+
     sub open_r {
-        my ($self, @rest) = @_;
-        unless (ref($self)) {
-            $self = $self->new(shift @rest);
-        }
-        $self->open('<:utf8', @rest);
+        ref($_[0]) || shift(@_);
+        my ($self, @args) = @_;
+        Sidef::Types::Glob::File::open($self, '<:utf8', @args);
     }
 
     *open_read = \&open_r;
 
     sub open_w {
-        my ($self, @rest) = @_;
-        unless (ref($self)) {
-            $self = $self->new(shift @rest);
-        }
-        $self->open('>:utf8', @rest);
+        ref($_[0]) || shift(@_);
+        my ($self, @args) = @_;
+        Sidef::Types::Glob::File::open($self, '>:utf8', @args);
     }
 
     *open_write = \&open_w;
 
     sub open_a {
-        my ($self, @rest) = @_;
-        unless (ref($self)) {
-            $self = $self->new(shift @rest);
-        }
-        $self->open('>>:utf8', @rest);
+        ref($_[0]) || shift(@_);
+        my ($self, @args) = @_;
+        Sidef::Types::Glob::File::open($self, '>>:utf8', @args);
     }
 
     *open_append = \&open_a;
 
     sub open_rw {
-        my ($self, @rest) = @_;
-        unless (ref($self)) {
-            $self = $self->new(shift @rest);
-        }
-        $self->open('+<:utf8', @rest);
+        ref($_[0]) || shift(@_);
+        my ($self, @args) = @_;
+        Sidef::Types::Glob::File::open($self, '+<:utf8', @args);
     }
 
     *open_read_write = \&open_rw;
 
     sub opendir {
         ref($_[0]) || shift(@_);
-        my ($self, @rest) = @_;
-        Sidef::Types::Glob::Dir->new("$self")->open(@rest);
+        my ($self, @args) = @_;
+        Sidef::Types::Glob::Dir->new("$self")->open(@args);
     }
 
     sub sysopen {
@@ -494,7 +489,7 @@ package Sidef::Types::Glob::File {
         my $success = sysopen(my $fh, "$self", "$mode", $perm // 0666);
 
         if ($success) {
-            $$var_ref = Sidef::Types::Glob::FileHandle->new(fh => $fh, self => $self);
+            $$var_ref = Sidef::Types::Glob::FileHandle->new($fh, $self);
         }
 
         $success
@@ -555,12 +550,34 @@ package Sidef::Types::Glob::File {
               : (Sidef::Types::Bool::Bool::FALSE);
         }
         else {
-            Sidef::Types::Number::Number::_new_uint(CORE::unlink(@args));
+            Sidef::Types::Number::Number->_new_uint(CORE::unlink(@args));
         }
     }
 
-    *delete = \&unlink;
-    *remove = \&unlink;
+    sub delete {
+        my ($self, @args) = @_;
+
+        if (ref($self)) {
+            my $file = "$self";
+            CORE::unlink($file) || return Sidef::Types::Bool::Bool::FALSE;
+            1 while CORE::unlink($file);
+            return Sidef::Types::Bool::Bool::TRUE;
+        }
+        else {
+            my $count = 0;
+
+            foreach my $arg (@args) {
+                my $file = "$arg";
+                CORE::unlink($file) || next;
+                1 while CORE::unlink($file);
+                ++$count;
+            }
+
+            Sidef::Types::Number::Number->_new_uint($count);
+        }
+    }
+
+    *remove = \&delete;
 
     sub split {
         ref($_[0]) || shift(@_);
