@@ -29,6 +29,7 @@ package Sidef::Types::Number::Inf {
     }
 
     *iadd = \&add;
+    *fadd = \&add;
 
     sub sub {
         my ($x, $y) = @_;
@@ -36,6 +37,7 @@ package Sidef::Types::Number::Inf {
     }
 
     *isub = \&sub;
+    *fsub = \&sub;
 
     sub mul {
         my ($x, $y) = @_;
@@ -44,6 +46,7 @@ package Sidef::Types::Number::Inf {
     }
 
     *imul = \&mul;
+    *fmul = \&mul;
 
     sub div {
         my ($x, $y) = @_;
@@ -54,6 +57,7 @@ package Sidef::Types::Number::Inf {
     }
 
     *idiv = \&div;
+    *fdiv = \&div;
 
     sub is_pos { Sidef::Types::Bool::Bool::TRUE }
 
@@ -101,6 +105,10 @@ package Sidef::Types::Number::Inf {
     *next_pow    = \&nan;
     *next_power  = \&nan;
     *digit       = \&nan;
+    *bernreal    = \&nan;
+    *bernfrac    = \&nan;
+    *bern        = \&nan;
+    *valuation   = \&nan;
 
     sub as_bin {
         state $x = Sidef::Types::String::String->new;
@@ -146,6 +154,7 @@ package Sidef::Types::Number::Inf {
     *cbrt             = \&inf;
     *sqr              = \&inf;
     *log              = \&inf;
+    *lgrt             = \&inf;
     *log2             = \&inf;
     *log10            = \&inf;
     *exp              = \&inf;
@@ -165,7 +174,9 @@ package Sidef::Types::Number::Inf {
     *digamma          = \&inf;
     *lngamma          = \&inf;
     *eint             = \&inf;
-    *li2              = \&inf;
+    *ei               = \&inf;
+    *li2              = \&ninf;
+    *li               = \&inf;
     *inc              = \&inf;
     *dec              = \&inf;
     *int              = \&inf;
@@ -197,6 +208,7 @@ package Sidef::Types::Number::Inf {
     *grad2deg         = \&inf;
     *round            = \&inf;
     *roundf           = \&inf;
+    *popcount         = \&nan;
 
     sub zero { Sidef::Types::Number::Number::ZERO }
 
@@ -347,6 +359,8 @@ package Sidef::Types::Number::Inf {
           :               $x;
     }
 
+    *fpow = \&pow;
+
     sub ipow {
         $_[0]->pow($_[1]->int);
     }
