@@ -29,21 +29,24 @@ package Sidef::Types::Number::Nan {
     *abs         = \&nan;
     *add         = \&nan;
     *iadd        = \&nan;
+    *fadd        = \&nan;
     *sub         = \&nan;
     *isub        = \&nan;
+    *fsub        = \&nan;
     *sqrt        = \&nan;
     *isqrt       = \&nan;
     *mul         = \&nan;
     *imul        = \&nan;
+    *fmul        = \&nan;
     *div         = \&nan;
     *idiv        = \&nan;
-    *mod         = \&nan;
+    *fdiv        = \&nan;
     *inv         = \&nan;
-    *imod        = \&nan;
     *pow         = \&nan;
     *ipow        = \&nan;
-    *iroot       = \&nan;
+    *fpow        = \&nan;
     *root        = \&nan;
+    *iroot       = \&nan;
     *and         = \&nan;
     *or          = \&nan;
     *xor         = \&nan;
@@ -67,7 +70,9 @@ package Sidef::Types::Number::Nan {
     *log         = \&nan;
     *log2        = \&nan;
     *log10       = \&nan;
+    *lgrt        = \&nan;
     *mod         = \&nan;
+    *imod        = \&nan;
     *fmod        = \&nan;
     *ln          = \&nan;
     *exp         = \&nan;
@@ -103,7 +108,9 @@ package Sidef::Types::Number::Nan {
     *erf         = \&nan;
     *erfc        = \&nan;
     *eint        = \&nan;
+    *ei          = \&nan;
     *li2         = \&nan;
+    *li          = \&nan;
     *max         = \&nan;
     *min         = \&nan;
     *int         = \&nan;
@@ -133,6 +140,11 @@ package Sidef::Types::Number::Nan {
     *roundf      = \&nan;
     *re          = \&nan;
     *real        = \&nan;
+    *bernreal    = \&nan;
+    *bernfrac    = \&nan;
+    *bern        = \&nan;
+    *popcount    = \&nan;
+    *valuation   = \&nan;
 
     sub cmp { }
 
