@@ -27,6 +27,7 @@ package Sidef::Types::Number::Ninf {
     }
 
     *iadd = \&add;
+    *fadd = \&add;
 
     sub sub {
         my ($x, $y) = @_;
@@ -34,6 +35,7 @@ package Sidef::Types::Number::Ninf {
     }
 
     *isub = \&sub;
+    *fsub = \&sub;
 
     sub mul {
         my ($x, $y) = @_;
@@ -42,6 +44,7 @@ package Sidef::Types::Number::Ninf {
     }
 
     *imul = \&mul;
+    *fmul = \&mul;
 
     sub div {
         my ($x, $y) = @_;
@@ -52,6 +55,7 @@ package Sidef::Types::Number::Ninf {
     }
 
     *idiv = \&div;
+    *fdiv = \&div;
 
     sub is_pos { Sidef::Types::Bool::Bool::FALSE }
 
@@ -111,12 +115,18 @@ package Sidef::Types::Number::Ninf {
     *next_pow         = \&nan;
     *next_power       = \&nan;
     *digit            = \&nan;
+    *bernreal         = \&nan;
+    *bernfrac         = \&nan;
+    *bern             = \&nan;
+    *popcount         = \&nan;
+    *valuation        = \&nan;
 
     sub inf { Sidef::Types::Number::Inf::INF }
 
     *neg    = \&inf;
     *abs    = \&inf;
     *log    = \&inf;
+    *lgrt   = \&inf;
     *cosh   = \&inf;
     *acosh  = \&inf;
     *tan    = \&inf;
@@ -128,6 +138,7 @@ package Sidef::Types::Number::Ninf {
     *len    = \&inf;
     *size   = \&inf;
     *not    = \&inf;
+    *li     = \&inf;
 
     sub ninf { NINF }
 
@@ -172,6 +183,7 @@ package Sidef::Types::Number::Ninf {
     *acsc  = \&zero;
     *acsch = \&zero;
     *eint  = \&zero;
+    *ei    = \&zero;
     *exp   = \&zero;
     *exp2  = \&zero;
     *exp10 = \&zero;
@@ -283,7 +295,11 @@ package Sidef::Types::Number::Ninf {
           :               $x->neg;
     }
 
-    *ipow = \&pow;
+    *fpow = \&pow;
+
+    sub ipow {
+        $_[0]->pow($_[1]->int);
+    }
 
     #
     ## binomial(-inf, x) = 0        | with x < 0
