@@ -8,7 +8,7 @@ package Sidef::Types::Number::Number {
     use Math::MPFR qw();
 
     our $ROUND = Math::MPFR::MPFR_RNDN();
-    our $PREC  = 128;
+    our $PREC  = 200;
 
     my $ONE = Math::GMPq::Rmpq_init();
     Math::GMPq::Rmpq_set_ui($ONE, 1, 1);
@@ -46,7 +46,7 @@ package Sidef::Types::Number::Number {
         bless(\$_[0], __PACKAGE__);
     }
 
-    sub _dump {
+    sub _deparse {
         my $x = ${$_[0]};
 
         if (    Math::GMPq::Rmpq_integer_p($x)
@@ -68,8 +68,9 @@ package Sidef::Types::Number::Number {
     }
 
     sub _new_uint {
-        exists($cache[$_[1]])
-          && return $cache[$_[1]];
+        $_[1] <= 8192
+          and exists($cache[$_[1]])
+          and return $cache[$_[1]];
         my $r = Math::GMPq::Rmpq_init();
         Math::GMPq::Rmpq_set_ui($r, $_[1], 1);
         $_[1] <= 8192
@@ -87,9 +88,7 @@ package Sidef::Types::Number::Number {
 
     sub _set_str {
         my $r = Math::GMPq::Rmpq_init();
-        index($_[1], '/') == -1
-          ? Math::GMPq::Rmpq_set_str($r, "$_[1]/1", 10)
-          : Math::GMPq::Rmpq_set_str($r, $_[1],     10);
+        Math::GMPq::Rmpq_set_str($r, $_[1], 10);
         bless \$r, __PACKAGE__;
     }
 
@@ -116,13 +115,12 @@ package Sidef::Types::Number::Number {
                 return __PACKAGE__->new($num)->div(__PACKAGE__->new($den));
             }
 
-            eval { Math::GMPq::Rmpq_set_str($r, $rat, $base) };
+            # Set the string
+            eval { Math::GMPq::Rmpq_set_str($r, $rat, $base); 1 } // return nan();
 
-            # Return NaN in case of an error
-            $@ && return nan();
-
-            #$@ && die "[ERROR] Value <<$num>> is not a valid base-$base number";
+            # Canonicalize the fraction
             Math::GMPq::Rmpq_canonicalize($r) if $is_frac;
+
             bless \$r, __PACKAGE__;
           };
     }
@@ -166,17 +164,19 @@ package Sidef::Types::Number::Number {
 
     sub _mpfr2big {
 
-        if (Math::MPFR::Rmpfr_inf_p($_[0])) {
-            if (Math::MPFR::Rmpfr_sgn($_[0]) > 0) {
-                return state $x = inf();
+        if (!Math::MPFR::Rmpfr_number_p($_[0])) {
+            if (Math::MPFR::Rmpfr_inf_p($_[0])) {
+                if (Math::MPFR::Rmpfr_sgn($_[0]) > 0) {
+                    return state $x = inf();
+                }
+                else {
+                    return state $x = ninf();
+                }
             }
-            else {
-                return state $x = ninf();
-            }
-        }
 
-        if (Math::MPFR::Rmpfr_nan_p($_[0])) {
-            return state $x = nan();
+            if (Math::MPFR::Rmpfr_nan_p($_[0])) {
+                return state $x = nan();
+            }
         }
 
         my $r = Math::GMPq::Rmpq_init();
@@ -277,11 +277,8 @@ package Sidef::Types::Number::Number {
             Math::GMPq::Rmpq_set($n, $x);
             Math::GMPq::Rmpq_abs($n, $n) if $sgn < 0;
 
-            my $z = Math::GMPz::Rmpz_init();
-            Math::GMPz::Rmpz_ui_pow_ui($z, 10, CORE::abs($prec));
-
             my $p = Math::GMPq::Rmpq_init();
-            Math::GMPq::Rmpq_set_z($p, $z);
+            Math::GMPq::Rmpq_set_str($p, '1' . ('0' x CORE::abs($prec)), 10);
 
             if ($prec < 0) {
                 Math::GMPq::Rmpq_div($n, $n, $p);
@@ -296,6 +293,7 @@ package Sidef::Types::Number::Number {
                 $q;
             };
 
+            my $z = Math::GMPz::Rmpz_init();
             Math::GMPq::Rmpq_add($n, $n, $half);
             Math::GMPz::Rmpz_set_q($z, $n);
 
@@ -456,6 +454,14 @@ package Sidef::Types::Number::Number {
         _mpz2big($x);
     }
 
+    sub fadd {
+        my ($x, $y) = @_;
+        _valid(\$y);
+        $x = _big2mpfr($x);
+        Math::MPFR::Rmpfr_add($x, $x, _big2mpfr($y), $ROUND);
+        _mpfr2big($x);
+    }
+
     sub sub {
         my ($x, $y) = @_;
 
@@ -484,6 +490,14 @@ package Sidef::Types::Number::Number {
         _mpz2big($x);
     }
 
+    sub fsub {
+        my ($x, $y) = @_;
+        _valid(\$y);
+        $x = _big2mpfr($x);
+        Math::MPFR::Rmpfr_sub($x, $x, _big2mpfr($y), $ROUND);
+        _mpfr2big($x);
+    }
+
     sub mul {
         my ($x, $y) = @_;
 
@@ -513,6 +527,14 @@ package Sidef::Types::Number::Number {
         _mpz2big($x);
     }
 
+    sub fmul {
+        my ($x, $y) = @_;
+        _valid(\$y);
+        $x = _big2mpfr($x);
+        Math::MPFR::Rmpfr_mul($x, $x, _big2mpfr($y), $ROUND);
+        _mpfr2big($x);
+    }
+
     sub div {
         my ($x, $y) = @_;
 
@@ -538,6 +560,14 @@ package Sidef::Types::Number::Number {
         bless \$r, __PACKAGE__;
     }
 
+    sub fdiv {
+        my ($x, $y) = @_;
+        _valid(\$y);
+        $x = _big2mpfr($x);
+        Math::MPFR::Rmpfr_div($x, $x, _big2mpfr($y), $ROUND);
+        _mpfr2big($x);
+    }
+
     sub idiv {
         my ($x, $y) = @_;
         _valid(\$y);
@@ -641,7 +671,7 @@ package Sidef::Types::Number::Number {
         }
 
         _valid(\$y);
-        return $x->pow($y->inv);
+        $x->pow($y->inv);
     }
 
     sub iroot {
@@ -657,16 +687,25 @@ package Sidef::Types::Number::Number {
         _valid(\$y);
 
         $x = _big2mpz($x);
+
         my $root = CORE::int(Math::GMPq::Rmpq_get_d($$y));
 
-        my ($is_even, $is_neg) = $root % 2 == 0;
-        ($is_neg = Math::GMPz::Rmpz_sgn($x) < 0) if $is_even;
-        Math::GMPz::Rmpz_abs($x, $x) if ($is_even && $is_neg);
-        Math::GMPz::Rmpz_root($x, $x, $root);
+        if ($root == 0) {
+            Math::GMPz::Rmpz_sgn($x) || return ZERO;    # 0^Inf = 0
+            Math::GMPz::Rmpz_cmpabs_ui($x, 1) == 0 and return ONE;    # 1^Inf = 1 ; (-1)^Inf = 1
+            return inf();
+        }
+        elsif ($root < 0) {
+            my $sign = Math::GMPz::Rmpz_sgn($x) || return inf();      # 1 / 0^k = Inf
+            Math::GMPz::Rmpz_cmp_ui($x, 1) == 0 and return ONE;       # 1 / 1^k = 1
+            return $sign < 0 ? nan() : ZERO;
+        }
+        elsif ($root % 2 == 0 and Math::GMPz::Rmpz_sgn($x) < 0) {
+            return nan();
+        }
 
-        $is_even && $is_neg
-          ? Sidef::Types::Number::Complex->new(0, _mpz2big($x))
-          : _mpz2big($x);
+        Math::GMPz::Rmpz_root($x, $x, $root);
+        _mpz2big($x);
     }
 
     sub sqr {
@@ -694,28 +733,43 @@ package Sidef::Types::Number::Number {
 
         _valid(\$y);
 
-        if (Math::GMPq::Rmpq_integer_p($$x) and Math::GMPq::Rmpq_integer_p($$y)) {
+        if (Math::GMPq::Rmpq_integer_p($$y)) {
 
+            my $z   = Math::GMPz::Rmpz_init();
+            my $q   = Math::GMPq::Rmpq_init();
             my $pow = Math::GMPq::Rmpq_get_d($$y);
 
-            my $z = Math::GMPz::Rmpz_init();
-            Math::GMPz::Rmpz_set_q($z, $$x);
-            Math::GMPz::Rmpz_pow_ui($z, $z, CORE::abs($pow));
+            if (Math::GMPq::Rmpq_integer_p($$x)) {
+                Math::GMPz::Rmpz_set_q($z, $$x);
+                Math::GMPz::Rmpz_pow_ui($z, $z, CORE::abs($pow));
 
-            my $q = Math::GMPq::Rmpq_init();
-            Math::GMPq::Rmpq_set_z($q, $z);
+                Math::GMPq::Rmpq_set_z($q, $z);
 
-            if ($pow < 0) {
-                if (!Math::GMPq::Rmpq_sgn($q)) {
-                    return inf();
+                if ($pow < 0) {
+                    if (!Math::GMPq::Rmpq_sgn($q)) {
+                        return inf();
+                    }
+                    Math::GMPq::Rmpq_inv($q, $q);
                 }
-                Math::GMPq::Rmpq_inv($q, $q);
+            }
+            else {
+                Math::GMPq::Rmpq_numref($z, $$x);
+                Math::GMPz::Rmpz_pow_ui($z, $z, CORE::abs($pow));
+
+                Math::GMPq::Rmpq_set_num($q, $z);
+
+                Math::GMPq::Rmpq_denref($z, $$x);
+                Math::GMPz::Rmpz_pow_ui($z, $z, CORE::abs($pow));
+
+                Math::GMPq::Rmpq_set_den($q, $z);
+                Math::GMPq::Rmpq_canonicalize($q);
+
+                Math::GMPq::Rmpq_inv($q, $q) if $pow < 0;
             }
 
             return bless \$q, __PACKAGE__;
         }
-
-        if (Math::GMPq::Rmpq_sgn($$x) < 0 and !Math::GMPq::Rmpq_integer_p($$y)) {
+        elsif (Math::GMPq::Rmpq_sgn($$x) < 0) {
             return Sidef::Types::Number::Complex->new($x)->pow($y);
         }
 
@@ -724,6 +778,14 @@ package Sidef::Types::Number::Number {
         _mpfr2big($x);
     }
 
+    sub fpow {
+        my ($x, $y) = @_;
+        _valid(\$y);
+        $x = _big2mpfr($x);
+        Math::MPFR::Rmpfr_pow($x, $x, _big2mpfr($y), $ROUND);
+        _mpfr2big($x);
+    }
+
     sub ipow {
         my ($x, $y) = @_;
 
@@ -744,7 +806,7 @@ package Sidef::Types::Number::Number {
 
         if ($pow < 0) {
             return inf() if !Math::GMPz::Rmpz_sgn($x);
-            Math::GMPz::Rmpz_div($x, $ONE_Z, $x);
+            Math::GMPz::Rmpz_tdiv_q($x, $ONE_Z, $x);
         }
 
         _mpz2big($x);
@@ -822,6 +884,45 @@ package Sidef::Types::Number::Number {
         _mpfr2big($x);
     }
 
+    sub lgrt {
+        my ($x) = @_;
+
+        my $sgn = Math::GMPq::Rmpq_sgn($$x);
+
+        $sgn == 0 and return ninf();
+        Math::GMPq::Rmpq_cmp_ui($$x, 7, 10) < 0 and return Sidef::Types::Number::Complex->new($x)->lgrt;
+
+        my $d = _big2mpfr($x);
+        Math::MPFR::Rmpfr_log($d, $d, $ROUND);
+
+        my $p = Math::MPFR::Rmpfr_init2($PREC);
+        Math::MPFR::Rmpfr_ui_pow_ui($p, 10, CORE::int($PREC / 4), $ROUND);
+        Math::MPFR::Rmpfr_ui_div($p, 1, $p, $ROUND);
+
+        $x = Math::MPFR::Rmpfr_init2($PREC);
+        Math::MPFR::Rmpfr_set_ui($x, 1, $ROUND);
+
+        my $y = Math::MPFR::Rmpfr_init2($PREC);
+        Math::MPFR::Rmpfr_set_ui($y, 0, $ROUND);
+
+        my $tmp = Math::MPFR::Rmpfr_init2($PREC);
+
+        while (1) {
+            Math::MPFR::Rmpfr_sub($tmp, $x, $y, $ROUND);
+            Math::MPFR::Rmpfr_cmpabs($tmp, $p) <= 0 and last;
+
+            Math::MPFR::Rmpfr_set($y, $x, $ROUND);
+
+            Math::MPFR::Rmpfr_log($tmp, $x, $ROUND);
+            Math::MPFR::Rmpfr_add_ui($tmp, $tmp, 1, $ROUND);
+
+            Math::MPFR::Rmpfr_add($x, $x, $d, $ROUND);
+            Math::MPFR::Rmpfr_div($x, $x, $tmp, $ROUND);
+        }
+
+        _mpfr2big($x);
+    }
+
     sub exp {
         my $r = _big2mpfr($_[0]);
         Math::MPFR::Rmpfr_exp($r, $r, $ROUND);
@@ -1144,6 +1245,63 @@ package Sidef::Types::Number::Number {
         _mpfr2big($r);
     }
 
+    sub bernfrac {
+        my ($n) = @_;
+
+        $n = CORE::int(Math::GMPq::Rmpq_get_d($$n));
+
+        $n == 0 and return ONE;
+        $n > 1 and $n % 2 and return ZERO;    # Bn=0 for odd n>1
+        $n < 0 and return nan();
+
+        state $ONE_Z  = Math::GMPz::Rmpz_init_set_ui(1);
+        state $ZERO_Z = Math::GMPz::Rmpz_init_set_ui(0);
+
+        my @D = ($ZERO_Z, $ONE_Z, ($ZERO_Z) x $n);
+
+        my ($h, $w) = (1, 1);
+        foreach my $i (0 .. $n - 1) {
+            if ($w ^= 1) {
+                $D[$_] += $D[$_ - 1] for (1 .. $h - 1);
+            }
+            else {
+                $w = $h++;
+                $D[$w] += $D[$w + 1] while --$w;
+            }
+        }
+
+        my $den = Math::GMPz::Rmpz_init_set($ONE_Z);
+        Math::GMPz::Rmpz_mul_2exp($den, $den, $n + 1);
+        Math::GMPz::Rmpz_sub_ui($den, $den, 2);
+        Math::GMPz::Rmpz_neg($den, $den) if $n % 4 == 0;
+
+        my $r = Math::GMPq::Rmpq_init();
+        Math::GMPq::Rmpq_set_num($r, $D[$h - 1]);
+        Math::GMPq::Rmpq_set_den($r, $den);
+        Math::GMPq::Rmpq_canonicalize($r);
+
+        bless \$r, __PACKAGE__;
+    }
+
+    *bern = \&bernfrac;
+
+    sub bernreal {
+        my $r = _big2mpfr($_[0]);
+
+        Math::MPFR::Rmpfr_trunc($r, $r);
+        Math::MPFR::Rmpfr_zero_p($r)  && return ONE;
+        Math::MPFR::Rmpfr_sgn($r) < 0 && return nan();
+        Math::MPFR::Rmpfr_neg($r, $r, $ROUND);
+
+        my $zeta = Math::MPFR::Rmpfr_init2($PREC);
+        Math::MPFR::Rmpfr_set($zeta, $r, $ROUND);
+        Math::MPFR::Rmpfr_add_ui($zeta, $zeta, 1, $ROUND);
+        Math::MPFR::Rmpfr_zeta($zeta, $zeta, $ROUND);
+        Math::MPFR::Rmpfr_mul($r, $r, $zeta, $ROUND);
+
+        _mpfr2big($r);
+    }
+
     sub erf {
         my $r = _big2mpfr($_[0]);
         Math::MPFR::Rmpfr_erf($r, $r, $ROUND);
@@ -1162,6 +1320,15 @@ package Sidef::Types::Number::Number {
         _mpfr2big($r);
     }
 
+    *ei = \&eint;
+
+    sub li {
+        my $r = _big2mpfr($_[0]);
+        Math::MPFR::Rmpfr_log($r, $r, $ROUND);
+        Math::MPFR::Rmpfr_eint($r, $r, $ROUND);
+        _mpfr2big($r);
+    }
+
     sub li2 {
         my $r = _big2mpfr($_[0]);
         Math::MPFR::Rmpfr_li2($r, $r, $ROUND);
@@ -1363,6 +1530,12 @@ package Sidef::Types::Number::Number {
         }
     }
 
+    sub popcount {
+        my $z = _big2mpz($_[0]);
+        Math::GMPz::Rmpz_neg($z, $z) if Math::GMPz::Rmpz_sgn($z) < 0;
+        __PACKAGE__->_new_uint(Math::GMPz::Rmpz_popcount($z));
+    }
+
     sub is_int {
         my ($x) = @_;
         Math::GMPq::Rmpq_integer_p($$x)
@@ -1632,36 +1805,24 @@ package Sidef::Types::Number::Number {
 
     sub floor {
         my ($x) = @_;
+
         Math::GMPq::Rmpq_integer_p($$x) && return $x;
 
-        if (Math::GMPq::Rmpq_sgn($$x) > 0) {
-            my $z = Math::GMPz::Rmpz_init();
-            Math::GMPz::Rmpz_set_q($z, $$x);
-            _mpz2big($z);
-        }
-        else {
-            my $z = Math::GMPz::Rmpz_init();
-            Math::GMPz::Rmpz_set_q($z, $$x);
-            Math::GMPz::Rmpz_sub_ui($z, $z, 1);
-            _mpz2big($z);
-        }
+        my $z = Math::GMPz::Rmpz_init();
+        Math::GMPz::Rmpz_set_q($z, $$x);
+        Math::GMPz::Rmpz_sub_ui($z, $z, 1) if Math::GMPq::Rmpq_sgn($$x) < 0;
+        _mpz2big($z);
     }
 
     sub ceil {
         my ($x) = @_;
+
         Math::GMPq::Rmpq_integer_p($$x) && return $x;
 
-        if (Math::GMPq::Rmpq_sgn($$x) > 0) {
-            my $z = Math::GMPz::Rmpz_init();
-            Math::GMPz::Rmpz_set_q($z, $$x);
-            Math::GMPz::Rmpz_add_ui($z, $z, 1);
-            _mpz2big($z);
-        }
-        else {
-            my $z = Math::GMPz::Rmpz_init();
-            Math::GMPz::Rmpz_set_q($z, $$x);
-            _mpz2big($z);
-        }
+        my $z = Math::GMPz::Rmpz_init();
+        Math::GMPz::Rmpz_set_q($z, $$x);
+        Math::GMPz::Rmpz_add_ui($z, $z, 1) if Math::GMPq::Rmpq_sgn($$x) > 0;
+        _mpz2big($z);
     }
 
     sub inc {
@@ -1755,8 +1916,10 @@ package Sidef::Types::Number::Number {
     sub modpow {
         my ($x, $y, $z) = @_;
         _valid(\$y, \$z);
+        my $mod = _big2mpz($z);
+        Math::GMPz::Rmpz_sgn($mod) || return nan();
         $x = _big2mpz($x);
-        Math::GMPz::Rmpz_powm($x, $x, _big2mpz($y), _big2mpz($z));
+        Math::GMPz::Rmpz_powm($x, $x, _big2mpz($y), $mod);
         _mpz2big($x);
     }
 
@@ -1820,9 +1983,10 @@ package Sidef::Types::Number::Number {
 
     sub factorial {
         my ($x) = @_;
-        return nan() if Math::GMPq::Rmpq_sgn($$x) < 0;
+        my $n = CORE::int(Math::GMPq::Rmpq_get_d($$x));
+        return nan() if $n < 0;
         my $r = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_fac_ui($r, CORE::int(Math::GMPq::Rmpq_get_d($$x)));
+        Math::GMPz::Rmpz_fac_ui($r, $n);
         _mpz2big($r);
     }
 
@@ -1830,9 +1994,10 @@ package Sidef::Types::Number::Number {
 
     sub double_factorial {
         my ($x) = @_;
-        return nan() if Math::GMPq::Rmpq_sgn($$x) < 0;
+        my $n = CORE::int(Math::GMPq::Rmpq_get_d($$x));
+        return nan() if $n < 0;
         my $r = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_2fac_ui($r, CORE::int(Math::GMPq::Rmpq_get_d($$x)));
+        Math::GMPz::Rmpz_2fac_ui($r, $n);
         _mpz2big($r);
     }
 
@@ -1840,17 +2005,28 @@ package Sidef::Types::Number::Number {
 
     sub primorial {
         my ($x) = @_;
-        return nan() if Math::GMPq::Rmpq_sgn($$x) < 0;
+        my $n = CORE::int(Math::GMPq::Rmpq_get_d($$x));
+        return nan() if $n < 0;
         my $r = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_primorial_ui($r, CORE::int(Math::GMPq::Rmpq_get_d($$x)));
+        Math::GMPz::Rmpz_primorial_ui($r, $n);
+        _mpz2big($r);
+    }
+
+    sub lucas {
+        my ($x) = @_;
+        my $n = CORE::int(Math::GMPq::Rmpq_get_d($$x));
+        return nan() if $n < 0;
+        my $r = Math::GMPz::Rmpz_init();
+        Math::GMPz::Rmpz_lucnum_ui($r, $n);
         _mpz2big($r);
     }
 
     sub fibonacci {
         my ($x) = @_;
-        return nan() if Math::GMPq::Rmpq_sgn($$x) < 0;
+        my $n = CORE::int(Math::GMPq::Rmpq_get_d($$x));
+        return nan() if $n < 0;
         my $r = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_fib_ui($r, CORE::int(Math::GMPq::Rmpq_get_d($$x)));
+        Math::GMPz::Rmpz_fib_ui($r, $n);
         _mpz2big($r);
     }
 
@@ -1887,14 +2063,6 @@ package Sidef::Types::Number::Number {
         __PACKAGE__->_new_int(Math::GMPz::Rmpz_kronecker(_big2mpz($x), _big2mpz($y)));
     }
 
-    sub lucas {
-        my ($x) = @_;
-        return nan() if Math::GMPq::Rmpq_sgn($$x) < 0;
-        my $r = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_lucnum_ui($r, CORE::int(Math::GMPq::Rmpq_get_d($$x)));
-        _mpz2big($r);
-    }
-
     sub gcd {
         my ($x, $y) = @_;
         _valid(\$y);
@@ -1911,21 +2079,24 @@ package Sidef::Types::Number::Number {
         _mpz2big($x);
     }
 
-    # By default, the test is correct up to a maximum value of 341,550,071,728,320
-    # See: https://en.wikipedia.org/wiki/Miller%E2%80%93Rabin_primality_test#Deterministic_variants_of_the_test
+    sub valuation {
+        my ($x, $y) = @_;
+        _valid(\$y);
+        my $z = _big2mpz($y);
+        my $sgn = Math::GMPz::Rmpz_sgn($z) || return ZERO;
+        Math::GMPz::Rmpz_abs($z, $z) if $sgn < 0;
+        $x = _big2mpz($x);
+        __PACKAGE__->_new_uint(Math::GMPz::Rmpz_remove($x, $x, $z));
+    }
+
     sub is_prime {
         my ($x, $k) = @_;
-        if (
-            Math::GMPz::Rmpz_probab_prime_p(_big2mpz($x),
-                                            defined($k)
-                                            ? do { _valid(\$k); CORE::int Math::GMPq::Rmpq_get_d($$k) }
-                                            : 15) > 0
-          ) {
-            (Sidef::Types::Bool::Bool::TRUE);
-        }
-        else {
-            (Sidef::Types::Bool::Bool::FALSE);
-        }
+        (
+         Math::GMPz::Rmpz_probab_prime_p(_big2mpz($x),
+                                         defined($k) ? do { _valid(\$k); CORE::int Math::GMPq::Rmpq_get_d($$k) }
+                                         : 20) > 0
+          ) ? Sidef::Types::Bool::Bool::TRUE
+          : Sidef::Types::Bool::Bool::FALSE;
     }
 
     sub next_prime {
@@ -1956,21 +2127,51 @@ package Sidef::Types::Number::Number {
     *is_sqr = \&is_square;
 
     sub is_power {
-        my ($x) = @_;
+        my ($x, $y) = @_;
 
         if (!Math::GMPq::Rmpq_integer_p($$x)) {
             return (Sidef::Types::Bool::Bool::FALSE);
         }
 
+        if (defined $y) {
+            _valid(\$y);
+
+            Math::GMPq::Rmpq_equal($$x, $ONE)
+              && return Sidef::Types::Bool::Bool::TRUE;
+
+            my $pow = CORE::int(Math::GMPq::Rmpq_get_d($$y));
+
+            # Return a true value when $x=-1 and $y is odd
+            $pow % 2
+              and Math::GMPq::Rmpq_equal($$x, $MONE)
+              and return Sidef::Types::Bool::Bool::TRUE;
+
+            # Don't accept a non-positive power
+            # Also, when $x is negative and $y is even, return faster
+            if ($pow <= 0 or ($pow % 2 == 0 and Math::GMPq::Rmpq_sgn($$x) < 0)) {
+                return Sidef::Types::Bool::Bool::FALSE;
+            }
+
+            my $z = Math::GMPz::Rmpz_init();
+            my $r = Math::GMPz::Rmpz_init();
+            Math::GMPq::Rmpq_numref($z, $$x);
+            Math::GMPz::Rmpz_root($r, $z, $pow);
+            my $sgn = Math::GMPz::Rmpz_sgn($r) || return Sidef::Types::Bool::Bool::TRUE;    # $x was zero
+            Math::GMPz::Rmpz_neg($r, $r) if $sgn < 0;
+
+            return (
+                    (Math::GMPz::Rmpz_remove($z, $z, $r) == $pow and Math::GMPz::Rmpz_cmpabs_ui($z, 1) == 0)
+                    ? Sidef::Types::Bool::Bool::TRUE
+                    : Sidef::Types::Bool::Bool::FALSE
+                   );
+        }
+
         my $nz = Math::GMPz::Rmpz_init();
         Math::GMPq::Rmpq_numref($nz, $$x);
 
-        if (Math::GMPz::Rmpz_perfect_power_p($nz)) {
-            (Sidef::Types::Bool::Bool::TRUE);
-        }
-        else {
-            (Sidef::Types::Bool::Bool::FALSE);
-        }
+        Math::GMPz::Rmpz_perfect_power_p($nz)
+          ? Sidef::Types::Bool::Bool::TRUE
+          : Sidef::Types::Bool::Bool::FALSE;
     }
 
     *is_pow = \&is_power;
@@ -2120,11 +2321,8 @@ package Sidef::Types::Number::Number {
         Math::GMPq::Rmpq_set($n, $$x);
         Math::GMPq::Rmpq_abs($n, $n) if $sgn < 0;
 
-        my $z = Math::GMPz::Rmpz_init();
-        Math::GMPz::Rmpz_ui_pow_ui($z, 10, CORE::abs($nth));
-
         my $p = Math::GMPq::Rmpq_init();
-        Math::GMPq::Rmpq_set_z($p, $z);
+        Math::GMPq::Rmpq_set_str($p, '1' . ('0' x CORE::abs($nth)), 10);
 
         if ($nth < 0) {
             Math::GMPq::Rmpq_div($n, $n, $p);
@@ -2139,6 +2337,7 @@ package Sidef::Types::Number::Number {
             $q;
         };
 
+        my $z = Math::GMPz::Rmpz_init();
         Math::GMPq::Rmpq_add($n, $n, $half);
         Math::GMPz::Rmpz_set_q($z, $n);
 
