@@ -27,7 +27,7 @@ package Sidef::Types::Range::RangeString {
 
         if (not defined $to) {
             $to   = $from;
-            $from = Sidef::Types::Number::Number::_new_uint(CORE::ord("a"));
+            $from = Sidef::Types::Number::Number->_new_uint(CORE::ord("a"));
         }
 
         bless {
@@ -54,7 +54,7 @@ package Sidef::Types::Range::RangeString {
         }
         else {
             $to   = $from;
-            $from = Sidef::Types::Number::Number::_new_uint(CORE::ord("a"));
+            $from = Sidef::Types::Number::Number->_new_uint(CORE::ord("a"));
         }
 
         bless {
@@ -99,8 +99,8 @@ package Sidef::Types::Range::RangeString {
         my $from = $self->{from};
         my $to   = $self->{to};
 
-        my $asc = !!($step->is_pos);
-        my $i   = $from;
+        my $asc = ($self->{_asc} //= !!$step->is_pos);
+        my $i = $from;
 
         Sidef::Types::Block::Block->new(
             code => sub {
