@@ -1325,19 +1325,33 @@ package Sidef::Types::String::String {
         Sidef::Types::Array::Pair->new($_[0], $_[1]);
     }
 
-    sub basic_dump {
-        my ($self) = @_;
-        __PACKAGE__->new(q{'} . $$self =~ s{([\\'])}{\\$1}gr . q{'});
-    }
+    {
+        my %esc = (
+                   "\a"    => "\\a",
+                   "\b"    => "\\b",
+                   "\t"    => "\\t",
+                   "\n"    => "\\n",
+                   "\f"    => "\\f",
+                   "\r"    => "\\r",
+                   "\e"    => "\\e",
+                   chr(11) => "\\v",
+                  );
 
-    sub dump {
-        my ($self) = @_;
+        # Function by Gisle Aas, copied from `Data::Dump` (thanks).
+        sub dump {
+            my $str = "${$_[0]}";
 
-        state $x = eval { require Data::Dump };
-        $x || return $self->basic_dump;
+            $str =~ s/([\\\"]|#\{)/\\$1/g;
+            $str =~ /[^\040-\176]/ or return bless(\qq("$str"), __PACKAGE__);
 
-        local $Data::Dump::TRY_BASE64 = 0;
-        __PACKAGE__->new(Data::Dump::quote($$self) =~ s<(#\{)>{\\$1}gr);
+            $str =~ s/([\a\b\t\n\f\r\e\13])/$esc{$1}/g;
+            $str =~ s/([\0-\037])(?!\d)/CORE::sprintf('\\%o',CORE::ord($1))/eg;
+
+            $str =~ s/([\0-\037\177-\377])/CORE::sprintf('\\x%02X',CORE::ord($1))/eg;
+            $str =~ s/([^\040-\176])/CORE::sprintf('\\x{%X}',CORE::ord($1))/eg;
+
+            bless \qq("$str"), __PACKAGE__;
+        }
     }
 
     *inspect = \&dump;
